#region Using directives
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Serialization;

using Perception.Sheets;
using Perception;
using Perception.ILO.Engine;
#endregion

namespace MyFirstCSI
{
    /// <summary>
    /// The Sheet Control class, implementing the ISheet interface
    /// </summary>
    [SheetInfoRetriever(typeof(SheetControlInfoRetriever))]
    [SheetInformationProvider(typeof(SheetControlInfoProvider))]
    public partial class SheetControl : UserControl, ISheet
    {
        #region Members
        private string m_strUserName = Properties.Resources.IDS_USERNAME;
        private IProgram m_iProgram = null;
        private UIState m_UIState = UIState.Invisible;
        private InitializeState m_InitializeState = InitializeState.Unknown;
        private bool m_bDisposed = false;

        private bool m_bEditFlag;
        // private string m_strExampleValue = string.Empty;
        #endregion

        #region Constructor
        public SheetControl()
        {
            InitializeComponent();
            Disposed += SheetDisposed;
        }
        #endregion

        #region ISheet Members
        #region -> Events
        /// <summary>
        /// Occurs when the user name of the sheet is changed.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler UserNameChanged;

#pragma warning disable 067
        /// <summary>
        /// Occurs when one or more tool bar items are updated.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler ToolItemsUpdated;

        /// <summary>
        /// Occurs when the sheet asks to rebuild the menu and tool bar.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler RebuildDynamicMenuRequested;
#pragma warning restore 067
        #endregion

        #region -> Properties
        /// <summary>
        /// Gets/Sets the sheet user-name.
        /// </summary>
        public string UserName
        {
            get
            {
                // Return the current UserName
                return this.m_strUserName;
            }
            set
            {
                // Compare the new UserName to the current UserName
                if (String.Compare(this.m_strUserName, value) != 0)
                {
                    // Set the new UserName
                    this.m_strUserName = value;

                    // Fire the UserNameChanged event
                    this.UserNameChanged?.Invoke(this, new EventArgs());
                }
            }
        }

        /// <summary>
        /// Gets the name of the dynamic drop-down menu entry for the sheet.
        /// </summary>
        public string MenuName
        {
            get
            {
                if (String.IsNullOrEmpty(this.m_strUserName))
                {
                    // Return a string from the resource table when no current User Name is available
                    return Properties.Resources.IDS_USERNAME;
                }
                else
                {
                    // Return the current User Name
                    return this.m_strUserName;
                }
            }
        }

        /// <summary>
        /// Gets the icon of the sheet.
        /// </summary>
        public Icon Icon
        {
            get
            {
                return Properties.Resources.SheetIcon;
            }
        }

        /// <summary>
        /// Gets whether the delete may be activated. Application delete should delete the current active item on the sheet.
        /// </summary> 
        public bool DeleteActiveItemSupported
        {
            get
            {
                return ((listBox1.SelectedIndex >= 0) && (m_bEditFlag == false));
            }
        }

        /// <summary>
        /// Gets/Sets the user-interface state of the sheet.
        /// </summary>
        public UIState UIState
        {
            get
            {
                return this.m_UIState;
            }
            set
            {
                this.m_UIState = value;
                System.Diagnostics.Debug.WriteLine("UIState is now " + this.m_UIState.ToString());
            }
        }

        /// <summary>
        /// Gets the initialization state of the sheet.
        /// </summary>
        public InitializeState InitializeState
        {
            get
            {
                return this.m_InitializeState;
            }
        }
        #endregion

        #region -> Methods
        /// <summary>
        /// Update the items in the dynamic drop-down menu.
        /// </summary>
        public void UpdateMenuItems()
        {
        }

        /// <summary>
        /// Delete the actively selected item on the sheet.
        /// </summary>
        public void DeleteActiveItem()
        {
            listBox1.Items.Remove(listBox1.SelectedItem);
        }

        /// <summary>
        /// Perform steps to auto-configure the sheet.
        /// </summary>
        public void AutoConfig()
        {
            listBox1.Items.Clear();
            CtrlAcquisitionSystem AcqSys = CtrlAcquisitionSystemFactory.Create();
            CtrlGroup GroupAll = AcqSys.Groups.GroupAll;
            foreach (CtrlRecorder Recorder in GroupAll.Recorders)
            {
                listBox1.Items.Add(Recorder.Name);
            }
        }

        /// <summary>
        /// Initialize the sheet.
        /// </summary>
        /// <param name="iProgram">Interface to the program.</param>
        /// <returns>Returns <b>Succeeded</b> if the initialization is successful.</returns>
        public InitializeState Initialize(IProgram iProgram)
        {
            this.m_iProgram = iProgram;

            if (m_iProgram.UserMode == UserMode.Review)
            {
                this.m_InitializeState = InitializeState.NotAllowed;
                MessageBox.Show(this, "You cannot use this sheet in review mode.", "CSI Demo 1", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            else
            {
                this.m_InitializeState = InitializeState.Succeeded;
            }
            return this.m_InitializeState;
        }

        /// <summary>
        /// Perform steps that should be taken after a new settings file was loaded.
        /// </summary>
        public void PostLoad()
        {
        }

        /// <summary>
        /// Collect the dynamic drop-down menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic drop-down menu.</returns>
        public ToolStripItem[] GetDynamicMenu()
        {
            //
            // // Example code to show how to return the ToolStripItem array from you own ContextMenuStrip
            //
            // ContextMenuStrip strip = this.contextMenuStrip;
            // if (this.IsDisposed || this.Disposing || ContextMenuStrip == null)
            //    return null;
            //
            // // If needed update the ContextMenuStrip items
            //
            // ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            // strip.Items.CopyTo(Result, 0);
            // return Result
            return null;
        }

        /// <summary>
        /// Collect the dynamic tool bar menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic tool-bar menu.</returns>
        public ToolStripItem[] GetDynamicToolBar()
        {
            //
            // // Example code to show how to return the ToolStripItem array from you own Toolstrip
            // // Make sure that the Visible property of your Toolstrip has been set to false.
            //
            // ToolStrip strip = this.toolStrip;
            // if (this.IsDisposed || this.Disposing || strip == null)
            //    return null;
            //
            // // If needed update the ToolStrip items
            //
            // ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            // strip.Items.CopyTo(Result, 0);
            // return Result
            return null;
        }

        #endregion
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Load data from saved stream (from VWB).
        /// </summary>
        /// <param name="info">Stream from which to load</param>
        public SheetControl(SerializationInfo info, StreamingContext context)
             : this()
        {
            //
            // Example how to restore a persistent value from a stream
            //
            // m_strExampleValue = Tools.GetValue<string>(info, "ExampleValue", m_strExampleValue);
        }

        /// <summary>
        /// Save data into stream when saving VWB
        /// </summary>
        /// <param name="info">Stream in which to save</param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            //
            // Example how to save a persistent value from into a stream
            //
            // info.AddValue("ExampleValue", m_strExampleValue, typeof(string));
        }
        #endregion

        private void button1_Click(object sender, EventArgs e)
        {
            if (m_bEditFlag == false)
            {
                button1.Text = "En&ter";
                textBox1.Enabled = true;
                m_bEditFlag = true;
            }
            else
            {
                UserName = textBox1.Text;
                button1.Text = "Edi&t";
                textBox1.Enabled = false;
                m_bEditFlag = false;
                // Fire the RebuildDynamicMenuRequested event to update the dynamic menu name and toolbar
                if (this.RebuildDynamicMenuRequested != null)
                    this.RebuildDynamicMenuRequested(this, new EventArgs());
            }
        }

        private void SheetControl_Load(object sender, EventArgs e)
        {
            m_bEditFlag = false;
            textBox1.Text = m_strUserName;
            textBox1.Enabled = false;
        }

        #region Disposing
        /// <summary>
        /// Procedure to do your cleanup, this function is called from within the Dispose() method.
        /// </summary>
        private void SheetDisposed(object sender, EventArgs e)
        {
            if (IsDisposed)
                return;

            if (m_bDisposed)
                return;
            try
            {
                // Add your clean up code
            }
            catch
            {
            }
            m_bDisposed = true;
        }
        #endregion


    }
}