#region Using directives
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Serialization;

using Perception.Sheets;
using Perception;
using Perception.ILO.Engine;
using Perception.Common;
using Perception.Core.ExtensionMethods;
using Perception.ILO.Engine.Interfaces;
#endregion

namespace CSI_Acquisition
{
    /// <summary>
    /// The Sheet Control class, implementing the ISheet interface
    /// </summary>
    [SheetInfoRetriever(typeof(SheetControlInfoRetriever))]
    [SheetInformationProvider(typeof(SheetControlInfoProvider))]
    public partial class SheetControl : UserControl, ISheet
    {
        #region Members
        private string m_strUserName = Properties.Resources.IDS_USERNAME;
        private IProgram m_iProgram = null;
        private UIState m_UIState = UIState.Invisible;
        private InitializeState m_InitializeState = InitializeState.Unknown;
        private bool m_bDisposed = false;
        // private string m_strExampleValue = string.Empty;

        #region -> MyMembers

        private CtrlAcquisitionSystem m_MyDemoSystem = null;
        private CtrlGroup m_GroupAll = null;

        #endregion
        #endregion

        #region Constructor
        public SheetControl()
        {
            InitializeComponent();
            Disposed += SheetDisposed;
        }
        #endregion

        #region ISheet Members
        #region -> Events
        /// <summary>
        /// Occurs when the user name of the sheet is changed.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler UserNameChanged;

#pragma warning disable 067
        /// <summary>
        /// Occurs when one or more tool bar items are updated.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler ToolItemsUpdated;

        /// <summary>
        /// Occurs when the sheet asks to rebuild the menu and tool bar.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler RebuildDynamicMenuRequested;
#pragma warning restore 067
        #endregion

        #region -> Properties
        /// <summary>
        /// Gets/Sets the sheet user-name.
        /// </summary>
        public string UserName
        {
            get
            {
                // Return the current UserName
                return this.m_strUserName;
            }
            set
            {
                // Compare the new UserName to the current UserName
                if (String.Compare(this.m_strUserName, value) != 0)
                {
                    // Set the new UserName
                    this.m_strUserName = value;

                    // Fire the UserNameChanged event
                    this.UserNameChanged?.Invoke(this, new EventArgs());
                }
            }
        }

        /// <summary>
        /// Gets the name of the dynamic drop-down menu entry for the sheet.
        /// </summary>
        public string MenuName
        {
            get
            {
                if (String.IsNullOrEmpty(this.m_strUserName))
                {
                    // Return a string from the resource table when no current User Name is available
                    return Properties.Resources.IDS_USERNAME;
                }
                else
                {
                    // Return the current User Name
                    return this.m_strUserName;
                }
            }
        }

        /// <summary>
        /// Gets the icon of the sheet.
        /// </summary>
        public Icon Icon
        {
            get
            {
                return Properties.Resources.SheetIcon;
            }
        }

        /// <summary>
        /// Gets whether the delete may be activated. Application delete should delete the current active item on the sheet.
        /// </summary> 
        public bool DeleteActiveItemSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets/Sets the user-interface state of the sheet.
        /// </summary>
        public UIState UIState
        {
            get
            {
                return this.m_UIState;
            }
            set
            {
                this.m_UIState = value;
            }
        }

        /// <summary>
        /// Gets the initialization state of the sheet.
        /// </summary>
        public InitializeState InitializeState
        {
            get
            {
                return this.m_InitializeState;
            }
        }
        #endregion

        #region -> Methods
        /// <summary>
        /// Update the items in the dynamic drop-down menu.
        /// </summary>
        public void UpdateMenuItems()
        {
        }

        /// <summary>
        /// Delete the actively selected item on the sheet.
        /// </summary>
        public void DeleteActiveItem()
        {
        }

        /// <summary>
        /// Perform steps to auto-configure the sheet.
        /// </summary>
        public void AutoConfig()
        {
        }

        /// <summary>
        /// Initialize the sheet.
        /// </summary>
        /// <param name="iProgram">Interface to the program.</param>
        /// <returns>Returns <b>Succeeded</b> if the initialization is successful.</returns>
        public InitializeState Initialize(IProgram iProgram)
        {
            try
            {
                this.m_iProgram = iProgram;
                this.m_MyDemoSystem = CtrlAcquisitionSystemFactory.Create();
                m_GroupAll = m_MyDemoSystem.Groups.GroupAll;
                HookToGroupAll();
                this.m_InitializeState = InitializeState.Succeeded;
            }
            catch
            {
                this.m_InitializeState = InitializeState.NotAllowed;
                PerceptionMessageBox.Show("Could not initialize sheet", "CSI: Catch", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            return this.m_InitializeState;
        }

        private void HookToGroupAll()
        {
            UnHookFromGroupAll();
            if (m_GroupAll != null)
            {
                m_GroupAll.AcquisitionStateChanged += GroupAllAcquisitionStateChanged;
            }
        }

        private void UnHookFromGroupAll()
        {
            if (m_GroupAll != null)
            {
                try
                {
                    m_GroupAll.AcquisitionStateChanged -= GroupAllAcquisitionStateChanged;
                }
                catch
                {
                }
            }
        }

        void GroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            this.InvokeOnUI(() => DoGroupAllAcquisitionStateChanged(sender, Running, OneShot, Stopping, Paused, Idle));
        }

        void DoGroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            if (Running > 0)
            {
                AcqStatus.Text = "Active";
                StartCmd.Enabled = false;
                StopCmd.Enabled = true;
            }
            else if (Paused > 0)
            {
                AcqStatus.Text = "Paused";
                StartCmd.Enabled = true;
                StopCmd.Enabled = true;
            }
            else if (OneShot > 0)
            {
                AcqStatus.Text = "Single shot mode";
                StartCmd.Enabled = false;
                StopCmd.Enabled = true;
            }
            else if (Stopping > 0)
            {
                AcqStatus.Text = "Stopping";
                StartCmd.Enabled = false;
                StopCmd.Enabled = false;
            }
            else if (Idle > 0)
            {
                AcqStatus.Text = "Idle";
                StartCmd.Enabled = true;
                StopCmd.Enabled = false;
            }
            else
            {
                AcqStatus.Text = "****";
                StartCmd.Enabled = false;
                StopCmd.Enabled = false;
            }

            // Fire the ToolItemsUpdated event to update the toolbar
            this.ToolItemsUpdated?.Invoke(this, new EventArgs());
        }

        /// <summary>
        /// Perform steps that should be taken after a new settings file was loaded.
        /// </summary>
        public void PostLoad()
        {
        }

        /// <summary>
        /// Collect the dynamic drop-down menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic drop-down menu.</returns>
        public ToolStripItem[] GetDynamicMenu()
        {
            //
            // // Example code to show how to return the ToolStripItem array from you own ContextMenuStrip
            //
            // ContextMenuStrip strip = this.contextMenuStrip;
            // if (this.IsDisposed || this.Disposing || ContextMenuStrip == null)
            //    return null;
            //
            // // If needed update the ContextMenuStrip items
            //
            // ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            // strip.Items.CopyTo(Result, 0);
            // return Result
            return null;
        }

        /// <summary>
        /// Collect the dynamic tool bar menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic tool-bar menu.</returns>
        public ToolStripItem[] GetDynamicToolBar()
        {
            //
            // // Example code to show how to return the ToolStripItem array from you own Toolstrip
            // // Make sure that the Visible property of your Toolstrip has been set to false.
            //
            // ToolStrip strip = this.toolStrip;
            // if (this.IsDisposed || this.Disposing || strip == null)
            //    return null;
            //
            // // If needed update the ToolStrip items
            //
            // ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            // strip.Items.CopyTo(Result, 0);
            // return Result
            return null;
        }

        #endregion
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Load data from saved stream (from VWB).
        /// </summary>
        /// <param name="info">Stream from which to load</param>
        public SheetControl(SerializationInfo info, StreamingContext context)
             : this()
        {
            //
            // Example how to restore a persistent value from a stream
            //
            // m_strExampleValue = Tools.GetValue<string>(info, "ExampleValue", m_strExampleValue);
        }

        /// <summary>
        /// Save data into stream when saving VWB
        /// </summary>
        /// <param name="info">Stream in which to save</param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            //
            // Example how to save a persistent value from into a stream
            //
            // info.AddValue("ExampleValue", m_strExampleValue, typeof(string));
        }
        #endregion

        private void SheetControl_Load(object sender, EventArgs e)
        {
            StartCmd.Enabled = false;
            StopCmd.Enabled = false;
        }

        private void StartCmd_Click(object sender, EventArgs e)
        {
            // for demo purposes -> set some acquisition parameters here
            // High Sampling rate to 10kHz; Pre and Post trigger duration to 50ms
            foreach (CtrlGroup myGroup in m_MyDemoSystem.Groups)
            {
                myGroup.HighSamplingFrequency = 10_000;
                }

            IAcquisitionSetup setup = m_MyDemoSystem.AcquisitionSetup;
            setup.PostTriggerInSeconds = 0.05;
            setup.PreTriggerInSeconds = 0.05;

            m_GroupAll.Run();
            StartCmd.Enabled = false;
            StopCmd.Enabled = true;
        }

        private void StopCmd_Click(object sender, EventArgs e)
        {
            m_GroupAll.Stop();
            StartCmd.Enabled = true;
            StopCmd.Enabled = false;
        }

        #region Disposing
        /// <summary>
        /// Procedure to do your cleanup, this function is called from within the Dispose() method.
        /// </summary>
        private void SheetDisposed(object sender, EventArgs e)
        {
            if (IsDisposed)
                return;

            if (m_bDisposed)
                return;
            try
            {
                // Add your clean up code
                UnHookFromGroupAll();
                m_GroupAll = null;
                m_MyDemoSystem = null;
            }
            catch
            {
            }
            m_bDisposed = true;
        }
        #endregion
    }
}