#region Using directives
using System;
using System.Drawing;
using System.Windows.Forms;
using System.Runtime.Serialization;

using Perception.Sheets;
using Perception;
using Perception.Common;
using Perception.ILO.Engine;
using Perception.Core.ExtensionMethods;
using Perception.CSI.Support.Sheets;
using DataSrcManager;
using System.Runtime.InteropServices;
using Perception.FormulaDatabase;
using Perception.ILO.Engine.Interfaces;
#endregion

namespace HelmetTest
{
    /// <summary>
    /// The Sheet Control class, implementing the ISheet interface
    /// </summary>
    [SheetInfoRetriever(typeof(SheetControlInfoRetriever))]
    [SheetInformationProvider(typeof(SheetControlInfoProvider))]
    public partial class SheetControl : UserControl, ISheet
    {
        #region Members
        private string m_strUserName = Properties.Resources.IDS_USERNAME;
        private IProgram m_iProgram = null;
        private UIState m_UIState = UIState.Invisible;
        private InitializeState m_InitializeState = InitializeState.Unknown;
        private bool m_bDisposed = false;
        #region -> MyMembers
        private CtrlAcquisitionSystem m_MyDemoSystem = null;
        private CtrlGroup m_GroupAll = null;
        private CtrlRecorder m_TheRecorder = null;
        private DataManager m_ThisDataManager = null;
        private bool m_MyTest = false;
        private bool m_Retry = false;
        private PoolEntry m_DUTSerialPEntry = null;
        #endregion
        #endregion

        #region Constructor
        public SheetControl()
        {
            InitializeComponent();
            if (m_ThisDataManager == null)
            {
                m_ThisDataManager = new DataManager();
            }
            Disposed += SheetDisposed;
        }
        #endregion

        #region ISheet Members
        #region -> Events
        /// <summary>
        /// Occurs when the user name of the sheet is changed.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler UserNameChanged;

#pragma warning disable 067
        /// <summary>
        /// Occurs when one or more tool bar items are updated.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler ToolItemsUpdated;

        /// <summary>
        /// Occurs when the sheet asks to rebuild the menu and tool bar.
        /// </summary>
        /// <remarks>
        /// Note that you should fire this event yourself when creating custom sheets.
        /// </remarks>
        public event EventHandler RebuildDynamicMenuRequested;
#pragma warning restore 067
        #endregion

        #region -> Properties
        /// <summary>
        /// Gets/Sets the sheet user-name.
        /// </summary>
        public string UserName
        {
            get
            {
                // Return the current UserName
                return this.m_strUserName;
            }
            set
            {
                // Compare the new UserName to the current UserName
                if (String.Compare(this.m_strUserName, value) != 0)
                {
                    // Set the new UserName
                    this.m_strUserName = value;

                    // Fire the UserNameChanged event
                    this.UserNameChanged?.Invoke(this, new EventArgs());
                }
            }
        }

        /// <summary>
        /// Gets the name of the dynamic drop-down menu entry for the sheet.
        /// </summary>
        public string MenuName
        {
            get
            {
                if (String.IsNullOrEmpty(this.m_strUserName))
                {
                    // Return a string from the resource table when no current User Name is available
                    return Properties.Resources.IDS_USERNAME;
                }
                else
                {
                    // Return the current User Name
                    return this.m_strUserName;
                }
            }
        }

        /// <summary>
        /// Gets the icon of the sheet.
        /// </summary>
        public Icon Icon
        {
            get
            {
                return Properties.Resources.SheetIcon;
            }
        }

        /// <summary>
        /// Gets whether the delete may be activated. Application delete should delete the current active item on the sheet.
        /// </summary> 
        public bool DeleteActiveItemSupported
        {
            get
            {
                return false;
            }
        }

        /// <summary>
        /// Gets/Sets the user-interface state of the sheet.
        /// </summary>
        public UIState UIState
        {
            get
            {
                return this.m_UIState;
            }
            set
            {
                this.m_UIState = value;
                if (this.m_UIState == UIState.Active)
                {
                    StatusArea.Refresh();
                }
            }
        }

        /// <summary>
        /// Gets the initialization state of the sheet.
        /// </summary>
        public InitializeState InitializeState
        {
            get
            {
                return this.m_InitializeState;
            }
        }
        #endregion

        #region -> Methods
        /// <summary>
        /// Update the items in the dynamic drop-down menu.
        /// </summary>
        public void UpdateMenuItems()
        {
        }

        /// <summary>
        /// Delete the actively selected item on the sheet.
        /// </summary>
        public void DeleteActiveItem()
        {
        }

        /// <summary>
        /// Perform steps to auto-configure the sheet.
        /// </summary>
        public void AutoConfig()
        {
        }

        /// <summary>
        /// Initialize the sheet.
        /// </summary>
        /// <param name="iProgram">Interface to the program.</param>
        /// <returns>Returns <b>Succeeded</b> if the initialization is successful.</returns>
        public InitializeState Initialize(IProgram iProgram)
        {
            if (iProgram.UserMode == UserMode.Review)
            {
                this.m_InitializeState = InitializeState.NotAllowed;
            }
            else
            {
                try
                {
                    this.m_iProgram = iProgram;
                    RegisterComponents();
                    this.m_MyDemoSystem = CtrlAcquisitionSystemFactory.Create();
                    this.m_GroupAll = m_MyDemoSystem.Groups.GroupAll;
                    HookToGroupAll();
                    m_DUTSerialPEntry = m_ThisDataManager.PoolEntries["Active.Information.DUTSerial"];
                    HookToPoolEntries();
                    this.m_InitializeState = InitializeState.Succeeded;
                }
                catch
                {
                    this.m_InitializeState = InitializeState.NotAllowed;
                    PerceptionMessageBox.Show(this, "Could not initialize sheet", "CSI: Catch", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            return this.m_InitializeState;

        }

        private void HookToPoolEntries()
        {
            UnHookFromPoolEntries();
            if (m_DUTSerialPEntry != null)
            {
                m_DUTSerialPEntry.DataSourceChanged += DUTSerialPEntry_DataSourceChanged;
                m_DUTSerialPEntry.DataChanged += DUTSerialPEntry_DataChanged;
            }
        }

        private void UnHookFromPoolEntries()
        {
            if (m_DUTSerialPEntry != null)
            {
                try
                {
                    m_DUTSerialPEntry.DataSourceChanged -= DUTSerialPEntry_DataSourceChanged;
                    m_DUTSerialPEntry.DataChanged -= DUTSerialPEntry_DataChanged;
                }
                catch
                {
                }
            }
        }

        private void HookToGroupAll()
        {
            UnHookFromGroupAll();
            if (m_GroupAll != null)
            {
                m_GroupAll.AcquisitionStateChanged += GroupAllAcquisitionStateChanged;
                m_GroupAll.RecorderAdded += GroupAllRecorderAdded;
            }
        }

        private void UnHookFromGroupAll()
        {
            if (m_GroupAll != null)
            {
                try
                {
                    m_GroupAll.AcquisitionStateChanged -= GroupAllAcquisitionStateChanged;
                    m_GroupAll.RecorderAdded -= GroupAllRecorderAdded;
                }
                catch
                {
                }
            }
        }


        void DUTSerialPEntry_DataSourceChanged()
        {
            this.InvokeOnUI(() => DoDUTSerialPEntry_DataSourceChanged());
        }

        void DoDUTSerialPEntry_DataSourceChanged()
        {
            // the pool entry for the DUTSerial has been changed
            // was there a request that we should handle?
            if (m_Retry == false)
                return;

            // yes, do the standard test
            // correctly finished, but serial number?
            if ((m_DUTSerialPEntry.DataSource == null) || (m_DUTSerialPEntry.DataSource.Value == null))
            {
                StatusArea.Text = "Please create serial number entry DUTSerial";
                m_Retry = true;
            }
            else
            {
                // variable exists
                string sDUT = m_DUTSerialPEntry.DataSource.Value.ToString();

                // some example testing, make your own
                if (sDUT.Length < 8)
                {
                    StatusArea.Text = "Please enter a correct serial number";
                    m_Retry = true;
                }
                else
                {
                    StatusArea.Text = "Test completed for device: " + sDUT;
                    PrintCmd.Enabled = true;
                    PrintCmd_MenuItem.Enabled = true;
                    PrintCmd_ToolStripItem.Enabled = true;
                    m_Retry = false;
                }
            }
            m_MyTest = false;
        }

        void DUTSerialPEntry_DataChanged()
        {
            this.InvokeOnUI(() => DoDUTSerialPEntry_DataChanged());
        }

        void DoDUTSerialPEntry_DataChanged()
        {
            // the data in the pool entry DUTSerial has been changed
            // this value is always transferred to our status
            // we know the entry exists, so only verify value
            string sDUT = m_DUTSerialPEntry.DataSource.Value.ToString();

            // some example testing, make your own
            if (sDUT.Length < 8)
            {
                StatusArea.Text = "Please enter a correct serial number";
                m_Retry = true;
            }
            else
            {
                StatusArea.Text = "Test completed for device: " + sDUT;
                PrintCmd.Enabled = true;
                PrintCmd_MenuItem.Enabled = true;
                PrintCmd_ToolStripItem.Enabled = true;
                m_Retry = false;
            }
            m_MyTest = false;
        }

        void GroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            this.InvokeOnUI(() => DoGroupAllRecorderAdded(sender, Recorder));
        }

        void DoGroupAllRecorderAdded(object sender, CtrlRecorder Recorder)
        {
            // is this the recorder we want ?
            if (Recorder.Name != "Recorder A")
                return;

            // already defined ?
            if (m_TheRecorder != null)
                return;

            m_TheRecorder = Recorder;
            // this should be > 0, but you never know
            if (m_TheRecorder.Channels.Count == 0)
            {
                // error handling here
            }

            // create required formulas in formula database

            // create instance of formula database
            FormulaDB ForDB = FormulaDB.Instance;

            // add formulas at fixed location
            // compensate X, Y and Z with mean of first sweep
            ForDB.Formulas[10].Name = "X";
            ForDB.Formulas[11].Name = "Y";
            ForDB.Formulas[12].Name = "Z";
            ForDB.Formulas[10].Expression = "@Sweep(Active.Group1.Recorder_A.Ch_A1; 2) - @Mean(@Sweep(Active.Group1.Recorder_A.Ch_A1; 1))";
            ForDB.Formulas[11].Expression = "@Sweep(Active.Group1.Recorder_A.Ch_A2; 2) - @Mean(@Sweep(Active.Group1.Recorder_A.Ch_A2; 1))";
            ForDB.Formulas[12].Expression = "@Sweep(Active.Group1.Recorder_A.Ch_A3; 2) - @Mean(@Sweep(Active.Group1.Recorder_A.Ch_A3; 1))";

            // calculate the result
            ForDB.Formulas[14].Name = "Resultant";
            ForDB.Formulas[14].Expression = "@Sqrt(@Pow(Formula.X; 2) + @Pow(Formula.Y; 2) + @Pow(Formula.Z; 2))";

            // for the time being use these values as the reference values
            ForDB.Formulas[16].Name = "ResultMax";
            ForDB.Formulas[16].Expression = "@Max(Formula.Resultant)";
            ForDB.Formulas[17].Name = "Ref80";
            ForDB.Formulas[17].Expression = "0.5 * Formula.ResultMax";
            ForDB.Formulas[18].Name = "Ref60";
            ForDB.Formulas[18].Expression = "0.6 * Formula.ResultMax";

            // calculate time parameters
            ForDB.Formulas[20].Name = "T1";
            ForDB.Formulas[20].Expression = "@NextLvlCross(Formula.Resultant; 0; Formula.Ref80; 1)";
            ForDB.Formulas[21].Name = "T2";
            ForDB.Formulas[21].Expression = "@NextLvlCross(Formula.Resultant; Formula.T1; Formula.Ref80; -1)";
            ForDB.Formulas[22].Name = "deltaT_80";
            ForDB.Formulas[22].Expression = "Formula.T2 - Formula.T1";
            ForDB.Formulas[22].Units = "s";

            ForDB.Formulas[23].Name = "T3";
            ForDB.Formulas[23].Expression = "@NextLvlCross(Formula.Resultant; 0; Formula.Ref60; 1)";
            ForDB.Formulas[24].Name = "T4";
            ForDB.Formulas[24].Expression = "@NextLvlCross(Formula.Resultant; Formula.T3; Formula.Ref60; -1)";
            ForDB.Formulas[25].Name = "deltaT_60";
            ForDB.Formulas[25].Expression = "Formula.T4 - Formula.T3";
            ForDB.Formulas[25].Units = "s";

            // clear display
            for (int j = ResultDisplay.pDisplay.TimeDisplay.CtlLayout.Pages.Count; j > 0; j--)
            {
                ResultDisplay.pDisplay.TimeDisplay.CtlLayout.Pages[j].Delete();
            }
            ResultDisplay.pDisplay.TimeDisplay.AddPage().Activate();

            // connect these waveforms to the display

            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.Pages.ActivePage.Panes[1].Activate();
            string[] aPoolEntry = new string[] { "Formula.X" };
            ResultDisplay.pDisplay.AddDataSources(aPoolEntry);
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.ActiveTrace.TraceProp.PrimaryColor = 0x0000FF; // red
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.Pages.ActivePage.Panes.AddPane().Activate();
            aPoolEntry[0] = "Formula.Y";
            ResultDisplay.pDisplay.AddDataSources(aPoolEntry);
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.ActiveTrace.TraceProp.PrimaryColor = 0xFFFFFF; // white
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.Pages.ActivePage.Panes.AddPane().Activate();
            aPoolEntry[0] = "Formula.Z";
            ResultDisplay.pDisplay.AddDataSources(aPoolEntry);
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.ActiveTrace.TraceProp.PrimaryColor = 0xFFFF00; // blue
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.Pages.ActivePage.Panes.AddPane().Activate();
            aPoolEntry[0] = "Formula.Resultant";
            ResultDisplay.pDisplay.AddDataSources(aPoolEntry);
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.ActiveTrace.TraceProp.PrimaryColor = 0x00FFFF; // yellow

            // set display
            ResultDisplay.pDisplay.ReviewType = TimeView.ReviewModeType.ReviewModeType_Recording;
            // set pane height
            double[] PaneArray = new double[4] { 0.2, 0.2, 0.2, 0.4 };
            object PaneParams = PaneArray;
            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.ActivePage.Panes.SetPaneHeights(ref PaneParams);

        }

        void GroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            this.InvokeOnUI(() => DoGroupAllAcquisitionStateChanged(sender, Running, OneShot, Stopping, Paused, Idle));
        }

        void DoGroupAllAcquisitionStateChanged(object sender, int Running, int OneShot, int Stopping, int Paused, int Idle)
        {
            // for any acquisition state change disable print unless correctly finished
            PrintCmd.Enabled = false;
            PrintCmd_MenuItem.Enabled = false;
            PrintCmd_ToolStripItem.Enabled = false;

            if (Running > 0)
            {
                StatusArea.Text = "Acquisition: running";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            else if (OneShot > 0)
            {
                StatusArea.Text = "Acquisition: single shot";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            else if (Stopping > 0)
            {
                StatusArea.Text = "Acquisition: stopping";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            else if (Paused > 0)
            {
                StatusArea.Text = "Acquisition: pause";
                StartCmd.Enabled = true;
                StartCmd_MenuItem.Enabled = true;
                StartCmd_ToolStripItem.Enabled = true;
            }
            else if (Idle > 0)
            {
                if (m_MyTest == true)
                {
                    // correctly finished, but serial number?
                    if (m_ThisDataManager.PoolEntries["Active.Information.DUTSerial"].DataSource == null)
                    {
                        StatusArea.Text = "Please create serial number entry DUTSerial";
                        m_Retry = true;
                    }
                    else
                    {
                        string sDUT;
                        // variable exists
                        if (m_ThisDataManager.PoolEntries["Active.Information.DUTSerial"].DataSource.Value == null)
                            sDUT = string.Empty;
                        else
                            sDUT = m_ThisDataManager.PoolEntries["Active.Information.DUTSerial"].DataSource.Value.ToString();

                        // some example testing, make your own
                        if (sDUT.Length < 8)
                        {
                            StatusArea.Text = "Please enter a correct serial number";
                            m_Retry = true;
                        }
                        else
                        {
                            StatusArea.Text = "Test completed for device: " + sDUT;
                            PrintCmd.Enabled = true;
                            PrintCmd_MenuItem.Enabled = true;
                            PrintCmd_ToolStripItem.Enabled = true;
                        }
                    }
                    m_MyTest = false;
                }
                else
                {
                    StatusArea.Text = "Acquisition: idle";
                }
                StartCmd.Enabled = true;
                StartCmd_MenuItem.Enabled = true;
                StartCmd_ToolStripItem.Enabled = true;
            }
            else
            {
                StatusArea.Text = "****";
                StartCmd.Enabled = false;
                StartCmd_MenuItem.Enabled = false;
                StartCmd_ToolStripItem.Enabled = false;
            }
            // Fire the ToolItemsUpdated event to update the toolbar
            if (this.ToolItemsUpdated != null)
                this.ToolItemsUpdated(this, new EventArgs());
        }

        /// <summary>
        /// Perform steps that should be taken after a new settings file was loaded.
        /// </summary>
        public void PostLoad()
        {
        }

        /// <summary>
        /// Collect the dynamic drop-down menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic drop-down menu.</returns>
        public ToolStripItem[] GetDynamicMenu()
        {
            ContextMenuStrip strip = contextMenuStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        /// <summary>
        /// Collect the dynamic tool bar menu.
        /// </summary>
        /// <returns>A tool strip array containing the items in the dynamic tool-bar menu.</returns>
        public ToolStripItem[] GetDynamicToolBar()
        {
            ToolStrip strip = toolStrip1;
            if (this.IsDisposed || this.Disposing || strip == null)
                return null;
            ToolStripItem[] Result = new ToolStripItem[strip.Items.Count];
            strip.Items.CopyTo(Result, 0);
            return Result;
        }

        #endregion
        #endregion

        #region ISerializable Members
        /// <summary>
        /// Load data from saved stream (from VWB).
        /// </summary>
        /// <param name="info">Stream from which to load</param>
        public SheetControl(SerializationInfo info, StreamingContext context)
             : this()
        {
            //
            // Example how to restore a persistent value from a stream
            //
            // m_strExampleValue = Tools.GetValue<string>(info, "ExampleValue", m_strExampleValue);
        }

        /// <summary>
        /// Save data into stream when saving VWB
        /// </summary>
        /// <param name="info">Stream in which to save</param>
        public void GetObjectData(SerializationInfo info, StreamingContext context)
        {
            //
            // Example how to save a persistent value from into a stream
            //
            // info.AddValue("ExampleValue", m_strExampleValue, typeof(string));
        }
        #endregion

        private void SheetControl_Load(object sender, EventArgs e)
        {
            StatusArea.Text = "****";
            StartCmd.Enabled = false;
            StartCmd_MenuItem.Enabled = false;
            StartCmd_ToolStripItem.Enabled = false;
            PrintCmd.Enabled = false;
            PrintCmd_MenuItem.Enabled = false;
            PrintCmd_ToolStripItem.Enabled = false;
            ResultMeters.UserName = "No Values";
            m_MyTest = false;
        }

        private void StartCmd_Click(object sender, EventArgs e)
        {
            // set all required acquisition parameters - example
            StatusArea.Text = "Setting test parameters";
            IAcquisitionSetup setup = m_MyDemoSystem.AcquisitionSetup;
            if ((setup != null) && (m_TheRecorder != null))
            {
                m_TheRecorder.Group.HighSamplingFrequency = 10000;  // sample rate 10 kHz
                setup.SweepCount = 2;
                // m_TheRecorder.Group.SweepCountEnabled = true;       // enable multiple sweep
                // m_TheRecorder.Group.SweepCount = 2;                 // 2 sweeps
                // m_TheRecorder.Group.SweepLength = 4000;             // length = 4000 samples
                // m_TheRecorder.Group.TriggerPosition = 50;           // trigger position 50%
                setup.PreTriggerInSeconds  = 2000 / m_TheRecorder.Group.HighSamplingFrequency;
                setup.PostTriggerInSeconds = 2000 / m_TheRecorder.Group.HighSamplingFrequency;
                setup.PostTriggerMode = CtrlPostTriggerMode.Immediate;
            }
            // start acquisition
            m_TheRecorder.Group.Run();
            PrintCmd.Enabled = false;
            PrintCmd_MenuItem.Enabled = false;
            PrintCmd_ToolStripItem.Enabled = false;

            m_MyTest = true;
            m_Retry = false;

            ResultDisplay.pDisplay.TimeDisplay.CtlLayout.HorizontalCursors.Visible = false;
        }

        private void PrintCmd_Click(object sender, EventArgs e)
        {
            Reporting.Print();
        }

        private void RegisterComponents()
        {
            if (m_iProgram == null)
                return;
            if (ResultDisplay != null)
            {
                if (ResultDisplay.pDisplay != null)
                {
                    // Add the display to the component manager
                    m_iProgram.ComponentManager.Add(ResultDisplay.pDisplay);
                }
            }
        }

        private void UnRegisterComponents()
        {
            if (m_iProgram == null)
                return;
            if (ResultDisplay != null)
            {
                if (ResultDisplay.pDisplay != null)
                {
                    // Remove the display from the component manager
                    m_iProgram.ComponentManager.Remove(ResultDisplay.pDisplay);
                }
            }
        }

        #region Disposing
        /// <summary>
        /// Procedure to do your cleanup, this function is called from within the Dispose() method.
        /// </summary>
        private void SheetDisposed(object sender, EventArgs e)
        {
            if (IsDisposed)
                return;

            if (m_bDisposed)
                return;
            try
            {
                // Add your clean up code
                UnHookFromPoolEntries();

                UnRegisterComponents();
                UnHookFromGroupAll();
                m_DUTSerialPEntry = null;

                m_GroupAll = null;
                m_MyDemoSystem = null;
                m_TheRecorder = null;
                    m_ThisDataManager = null;
            }
            catch
            {
            }
            m_bDisposed = true;
        }
        #endregion


    }
}